/* LDVINTFC.C -- LonWorks standard network interface functions
 * Copyright (c) 1993 by Echelon Corporation.
 * All Rights Reserved.
 *
 * Date last modified: 10/7/93
 *
 * Example Standard LONWORKS network interface functions
 * This layer contains all the DOS dependencies of the host application
 * interface to the network device.
 */

/****************************** Include Files *******************************/

#include <io.h>                     /* For ioctl()                         */
#include <fcntl.h>                  /* Modes and flags for open()          */
#include <sys\types.h>
#include <sys\stat.h>               /* For open()                          */
#include <stdio.h>                  /* For perror()                        */
#ifdef _MSC_VER
    extern int ioctl( int handle, int func, void far * argdx, int argcx );
#endif

#include "ldvintfc.h"

/*
 ****************************************************************************
 * Network driver interface structure.  This structure is used to pass the
 * read, write, and register function pointers back via the ioctl() call.
 ****************************************************************************
 */

typedef struct {
    unsigned char ioctl_stat;       /* Status                               */
    LDVCode ( far *read_fn ) ( void far *msg_p, unsigned length );
                                    /* Read function pointer                */
    LDVCode ( far *write_fn ) ( void far *msg_p, unsigned length  );
                                    /* Write function pointer               */
    void ( far *register_fn ) ( int handle,
                             void ( interrupt far *callback ) ( void ) );
                                    /* Register function pointer            */
} direct_calls;

/*
 ****************************************************************************
 * Device table - stores resources associated with each open
 * network interface.
 ****************************************************************************
 */

#define NUM_DEVICES 8

typedef struct {
    int          DOS_handle;      // -1 means entry is unused
    direct_calls calls;
} device;

static device device_table[ NUM_DEVICES ] =
  { { -1 }, { -1 }, { -1 }, { -1 }, { -1 }, { -1 }, { -1 }, { -1 } };

/*
 ****************************************************************************
 * ldv_open() - open network interface device
 ****************************************************************************
 */

LNI ldv_open( const char * device_name ) {
    int         DOS_handle;
    LNI         LNI_handle;
    device      * device_ptr;

    /* find a free entry in the device table */
    for( LNI_handle = 0, device_ptr = device_table; LNI_handle < NUM_DEVICES;
        LNI_handle++, device_ptr++ ) {
        if( device_ptr->DOS_handle == -1 ) break;
    }
    if( LNI_handle >= NUM_DEVICES ) return -1;     // error - too many opens

    /* open the device with DOS */
    DOS_handle = open( device_name, O_RDWR | O_BINARY );
    if( DOS_handle < 0 ) {          // DOS error
        perror( "Error opening network driver" );
        return -1;
    }

    /* get direct calls entry points */
    if( ioctl( DOS_handle, 2, & device_ptr->calls, sizeof( direct_calls ) )
        < 0 ) {
        perror( "Error initializing network interface" );
        close( DOS_handle );
        return -1;                  // error calling ioctl
    }
    device_ptr->DOS_handle = DOS_handle;
    return LNI_handle;
}

/*
 ****************************************************************************
 * ldv_read() - read buffer from network interface
 ****************************************************************************
 */

LDVCode ldv_read( LNI LNI_handle, void * msg_p, unsigned length ) {
    device      * device_ptr;

    if( LNI_handle < 0 ) return LDV_NOT_OPEN;
    if( LNI_handle > NUM_DEVICES ) return LDV_INVALID_DEVICE_ID;
    device_ptr = &device_table[ LNI_handle ];
    if( device_ptr->DOS_handle == -1 ) return LDV_NOT_OPEN;
    return device_ptr->calls.read_fn( msg_p, length );
}

/*
 ****************************************************************************
 * ldv_write() - write buffer to network interface
 ****************************************************************************
 */

LDVCode ldv_write( LNI LNI_handle, void * msg_p, unsigned length ) {
    device      * device_ptr;

    if( LNI_handle < 0 ) return LDV_NOT_OPEN;
    if( LNI_handle > NUM_DEVICES ) return LDV_INVALID_DEVICE_ID;
    device_ptr = &device_table[ LNI_handle ];
    if( device_ptr->DOS_handle == -1 ) return LDV_NOT_OPEN;
    return device_ptr->calls.write_fn( msg_p, length );
}

/*
 ****************************************************************************
 * ldv_close() - close network interface
 ****************************************************************************
 */

LDVCode ldv_close( LNI LNI_handle ) {
    device      * device_ptr;

    if( LNI_handle < 0 ) return LDV_NOT_OPEN;
    if( LNI_handle > NUM_DEVICES ) return LDV_INVALID_DEVICE_ID;
    device_ptr = & device_table[ LNI_handle ];
    if( device_ptr->DOS_handle == -1 ) return LDV_NOT_OPEN;
    if( close( device_ptr->DOS_handle ) < 0 ) {
        perror( "Error closing device" );
        return LDV_DEVICE_ERR;
    }
    device_ptr->DOS_handle = -1;      // show it's free
    return LDV_OK;
}
